@@ -17,7 +17,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: io-sim.c,v 1.3 2006/02/10 06:25:37 mschimek Exp $ */
+/* $Id: io-sim.c,v 1.5 2006/04/29 05:55:35 mschimek Exp $ */
 
 #include <assert.h>
 #include <stdlib.h>
@@ -270,11 +270,6 @@ signal_u8			(const vbi_sampling_par *sp,
 {
 	unsigned int scan_lines;
 
-	if (!sp->synchronous) {
-		log ("Requires synchronous sampling");
-		return FALSE;
-	}
-
 	scan_lines = sp->count[0] + sp->count[1];
 
 	clear_image (raw, blank_level,
@@ -288,7 +283,8 @@ signal_u8			(const vbi_sampling_par *sp,
 
 		if (0 == sliced->line) {
 			goto bounds;
-		} else if ((int) sliced->line >= sp->start[1]) {
+		} else if (0 != sp->start[1]
+			   && (int) sliced->line >= sp->start[1]) {
 			row = sliced->line - sp->start[1];
 
 			if (row >= sp->count[1])
@@ -298,7 +294,8 @@ signal_u8			(const vbi_sampling_par *sp,
 				row = row * 2 + 1;
 			else
 				row += sp->count[0];
-		} else if ((int) sliced->line >= sp->start[0]) {
+		} else if (0 != sp->start[0]
+			   && (int) sliced->line >= sp->start[0]) {
 			row = sliced->line - sp->start[0];
 
 			if (row >= sp->count[0])
@@ -308,7 +305,7 @@ signal_u8			(const vbi_sampling_par *sp,
 				row *= 2;
 		} else {
 		bounds:
-			log ("Sliced line %u out of bounds",
+			log ("Sliced line %u out of bounds\n",
 			     sliced->line);
 			return FALSE;
 		}
@@ -387,7 +384,7 @@ signal_u8			(const vbi_sampling_par *sp,
 			break;
 
 		default:
-			log ("Service 0x%08x (%s) not supported",
+			log ("Service 0x%08x (%s) not supported\n",
 			     sliced->id, vbi_sliced_name (sliced->id));
 			return FALSE;
 		}
@@ -429,7 +426,8 @@ _vbi_test_image_vbi		(uint8_t *		raw,
 	unsigned int black_level;
 	unsigned int white_level;
 
-	if (!_vbi_sampling_par_verify (sp))
+	if (!_vbi_sampling_par_valid (sp, vbi_log_on_stderr,
+				      /* log_user_data */ NULL))
 		return FALSE;
 
 	scan_lines = sp->count[0] + sp->count[1];
@@ -581,7 +579,8 @@ _vbi_test_image_video		(uint8_t *		raw,
 	uint8_t *s;
 	uint8_t *d;
 
-	if (!_vbi_sampling_par_verify (sp))
+	if (!_vbi_sampling_par_valid (sp, /* log_fn */ NULL,
+				      /* log_user_data */ NULL))
 		return FALSE;
 
 	scan_lines = sp->count[0] + sp->count[1];
