@@ -17,7 +17,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: raw_decoder.c,v 1.6 2006/02/10 06:25:37 mschimek Exp $ */
+/* $Id: raw_decoder.c,v 1.8 2006/04/29 05:55:35 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -46,11 +46,17 @@
 #  define PRId64 "lld"
 #endif
 
-#define log(templ, args...)						\
+#define sp_log(level, templ, args...)					\
 do {									\
-	if (RAW_DECODER_LOG)						\
-		fprintf (stderr, "%s: " templ ".\n",			\
-			 __FUNCTION__ , ##args);			\
+	vbi_log_printf (log_fn, log_user_data,				\
+			level, __FUNCTION__, templ , ##args);		\
+} while (0)
+
+#define rd_log(level, templ, args...)					\
+do {									\
+	if (level <= rd->log_max_level)					\
+		vbi_log_printf (rd->log_fn, rd->log_user_data,		\
+				level, __FUNCTION__, templ , ##args);	\
 } while (0)
 
 /**
@@ -73,7 +79,8 @@ _vbi_service_table [] = {
 		{ 6, 318 },
 		{ 22, 335 },
 		10500, 6203125, 6203125, /* 397 x FH */
-		0x00AAAAE7, 0xFFFF, 18, 6, 37 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE7, 0xFFFF, 18, 6, 37 * 8, VBI_MODULATION_NRZ_LSB,
+		0, /* probably */
 	}, {
 		VBI_SLICED_TELETEXT_B_L10_625,
 		"Teletext System B 625 Level 1.5",
@@ -81,7 +88,8 @@ _vbi_service_table [] = {
 		{ 7, 320 },
 		{ 22, 335 },
 		10300, 6937500, 6937500, /* 444 x FH */
-		0x00AAAAE4, 0xFFFF, 18, 6, 42 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE4, 0xFFFF, 18, 6, 42 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_B,
 		"Teletext System B, 625",
@@ -89,7 +97,8 @@ _vbi_service_table [] = {
 		{ 6, 318 },
 		{ 22, 335 },
 		10300, 6937500, 6937500, /* 444 x FH */
-		0x00AAAAE4, 0xFFFF, 18, 6, 42 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE4, 0xFFFF, 18, 6, 42 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_C_625, /* UNTESTED */
 		"Teletext System C 625",
@@ -97,7 +106,8 @@ _vbi_service_table [] = {
 		{ 6, 318 },
 		{ 22, 335 },
 		10480, 5734375, 5734375, /* 367 x FH */
-		0x00AAAAE7, 0xFFFF, 18, 6, 33 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE7, 0xFFFF, 18, 6, 33 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_D_625, /* UNTESTED */
 		"Teletext System D 625",
@@ -106,21 +116,24 @@ _vbi_service_table [] = {
 		{ 22, 335 },
 		10500, /* or 10970 depending on field order */
 		5642787, 5642787, /* 14/11 x FSC (color subcarrier) */
-		0x00AAAAE5, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE5, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_VPS, "Video Program System",
 		VBI_VIDEOSTD_SET_PAL_BG,
 		{ 16, 0 },
 		{ 16, 0 },
 		12500, 5000000, 2500000, /* 160 x FH */
-		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8, VBI_MODULATION_BIPHASE_MSB
+		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8, VBI_MODULATION_BIPHASE_MSB,
+		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_VPS_F2, "Pseudo-VPS on field 2",
 		VBI_VIDEOSTD_SET_PAL_BG,
 		{ 0, 329 },
 		{ 0, 329 },
 		12500, 5000000, 2500000, /* 160 x FH */
-		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8, VBI_MODULATION_BIPHASE_MSB
+		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8, VBI_MODULATION_BIPHASE_MSB,
+		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_WSS_625, "Wide Screen Signalling 625",
 		VBI_VIDEOSTD_SET_625_50,
@@ -128,28 +141,33 @@ _vbi_service_table [] = {
 		{ 23, 0 },
 		11000, 5000000, 833333, /* 160/3 x FH */
 		0xC71E3C1F, 0x924C99CE, 32, 0, 14 * 1,
-		VBI_MODULATION_BIPHASE_LSB
+		VBI_MODULATION_BIPHASE_LSB,
+		/* Hm. Easily confused with caption?? */
+		_VBI_SP_FIELD_NUM | _VBI_SP_LINE_NUM,
 	}, {
 		VBI_SLICED_CAPTION_625_F1, "Closed Caption 625, field 1",
 		VBI_VIDEOSTD_SET_625_50,
 		{ 22, 0 },
 		{ 22, 0 },
 		10500, 1000000, 500000, /* 32 x FH */
-		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB
+		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_CAPTION_625_F2, "Closed Caption 625, field 2",
 		VBI_VIDEOSTD_SET_625_50,
 		{ 0, 335 },
 		{ 0, 335 },
 		10500, 1000000, 500000, /* 32 x FH */
-		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB
+		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_VBI_625, "VBI 625", /* Blank VBI */
 		VBI_VIDEOSTD_SET_625_50,
 		{ 6, 318 },
 		{ 22, 335 },
 		10000, 1510000, 1510000,
-		0, 0, 0, 0, 10 * 8, 0 /* 10.0-2 ... 62.9+1 us */
+		0, 0, 0, 0, 10 * 8, 0, /* 10.0-2 ... 62.9+1 us */
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_B_525, /* UNTESTED */
 		"Teletext System B 525",
@@ -157,7 +175,8 @@ _vbi_service_table [] = {
 		{ 10, 272 },
 		{ 21, 284 },
 		10500, 5727272, 5727272, /* 364 x FH */
-		0x00AAAAE4, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE4, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_C_525, /* UNTESTED */
 		"Teletext System C 525",
@@ -165,7 +184,8 @@ _vbi_service_table [] = {
 		{ 10, 272 },
 		{ 21, 284 },
 		10480, 5727272, 5727272, /* 364 x FH */
-		0x00AAAAE7, 0xFFFF, 18, 6, 33 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE7, 0xFFFF, 18, 6, 33 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_D_525, /* UNTESTED */
 		"Teletext System D 525",
@@ -173,7 +193,8 @@ _vbi_service_table [] = {
 		{ 10, 272 },
 		{ 21, 284 },
 		9780, 5727272, 5727272, /* 364 x FH */
-		0x00AAAAE5, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE5, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 #if 0 /* FIXME probably wrong */
 		VBI_SLICED_WSS_CPR1204,	/* NOT CONFIRMED (EIA-J CPR-1204) */
@@ -182,8 +203,9 @@ _vbi_service_table [] = {
 		{ 20, 283 },
 		{ 20, 283 },
 		11200, 1789773, 447443, /* 1/8 x FSC */
-		0x000000F0, 0xFF, 8, 0, 20 * 1, VBI_MODULATION_NRZ_MSB
+		0x000000F0, 0xFF, 8, 0, 20 * 1, VBI_MODULATION_NRZ_MSB,
 		/* No useful FRC, but a six bit CRC */
+		0,
 	}, {
 #endif
 		VBI_SLICED_CAPTION_525_F1,
@@ -192,7 +214,10 @@ _vbi_service_table [] = {
 		{ 21, 0 },
 		{ 21, 0 },
 		10500, 1006976, 503488, /* 32 x FH */
-		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB
+		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		/* I've seen CC signals on other lines and there's no
+		   way to distinguish from the transmitted data. */
+		_VBI_SP_FIELD_NUM | _VBI_SP_LINE_NUM,
 	}, {
 		VBI_SLICED_CAPTION_525_F2,
 		"Closed Caption 525, field 2",
@@ -200,7 +225,8 @@ _vbi_service_table [] = {
 		{ 0, 284 },
 		{ 0, 284 },
 		10500, 1006976, 503488, /* 32 x FH */
-		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB
+		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		_VBI_SP_FIELD_NUM | _VBI_SP_LINE_NUM,
 	}, {
 		VBI_SLICED_2xCAPTION_525, /* NOT CONFIRMED */
 		"2xCaption 525",
@@ -209,21 +235,24 @@ _vbi_service_table [] = {
 		{ 21, 0 },
 		10500, 1006976, 1006976, /* 64 x FH */
 		0x000554ED, 0xFFFF, 12, 8, 4 * 8,
-		VBI_MODULATION_NRZ_LSB /* Tb. */
+		VBI_MODULATION_NRZ_LSB, /* Tb. */
+		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_VBI_525, "VBI 525", /* Blank VBI */
 		VBI_VIDEOSTD_SET_525_60,
 		{ 10, 272 },
 		{ 21, 284 },
 		9500, 1510000, 1510000,
-		0, 0, 0, 0, 10 * 8, 0 /* 9.5-1 ... 62.4+1 us */
+		0, 0, 0, 0, 10 * 8, 0, /* 9.5-1 ... 62.4+1 us */
+		0,
 	}, {
 		0, NULL,
 		VBI_VIDEOSTD_SET_EMPTY,
 		{ 0, 0 },
 		{ 0, 0 },
 		0, 0, 0,
-		0, 0, 0, 0, 0, 0
+		0, 0, 0, 0, 0, 0,
+		0,
 	}
 };
 
@@ -449,17 +478,16 @@ decode_pattern			(vbi3_raw_decoder *	rd,
 			/* Positive match, output decoded line. */
 
 			sliced->id = job->id;
+			sliced->line = 0;
 
 			if (i >= (unsigned int) sp->count[0]) {
-				if (0 == sp->start[1])
-					sliced->line = 0;
-				else
+				if (sp->synchronous
+				    && 0 != sp->start[1])
 					sliced->line = sp->start[1]
 						+ i - sp->count[0];
 			} else {
-				if (0 == sp->start[0])
-					sliced->line = 0;
-				else
+				if (sp->synchronous
+				    && 0 != sp->start[0])
 					sliced->line = sp->start[0] + i;
 			}
 
@@ -585,7 +613,9 @@ vbi3_raw_decoder_decode		(vbi3_raw_decoder *	rd,
 }
 
 vbi_bool
-_vbi_sampling_par_verify	(const vbi_sampling_par *sp)
+_vbi_sampling_par_valid		(const vbi_sampling_par *sp,
+				 vbi_log_fn *		log_fn,
+				 void *			log_user_data)
 {
 	switch (sp->sampling_format) {
 	case VBI_PIXFMT_YUV420:
@@ -626,31 +656,40 @@ _vbi_sampling_par_verify	(const vbi_sampling_par *sp)
 			    || (sp->start[1] + sp->count[0]) > 626)
 				goto range;
 	} else {
-		log ("Ambiguous scanning %d",
-		     sp->scanning);
+		sp_log (VBI_LOG_ERR,
+			"Ambiguous scanning %d.",
+			sp->scanning);
 		return FALSE;
 	}
 
 	if (sp->interlaced
-	    && sp->count[0] != sp->count[1]) {
-		log ("Line count %u, %u must be equal and non-zero "
-		     "when interlaced",
-		     sp->count[0], sp->count[1]);
+	    && (sp->count[0] != sp->count[1]
+		|| 0 == sp->count[0])) {
+		sp_log (VBI_LOG_ERR,
+			"Line counts %u, %u must be equal and "
+			"non-zero when raw VBI data is interlaced.",
+			sp->count[0], sp->count[1]);
 		return FALSE;
 	}
 
 	return TRUE;
 
  samples:
-	log ("Misaligned bytes_per_line %u",
-	     sp->bytes_per_line);
-
+	sp_log (VBI_LOG_ERR,
+		"bytes_per_line value %u is no multiple of "
+		"the sample size %u.",
+		sp->bytes_per_line,
+		VBI_PIXFMT_BPP (sp->sampling_format));
 	return FALSE;
 
  range:
-	log ("Invalid scan range %u-%u (%u lines), %u-%u (%u lines)",
-	     sp->start[0], sp->start[0] + sp->count[0] - 1, sp->count[0],
-	     sp->start[1], sp->start[1] + sp->count[1] - 1, sp->count[1]);
+	sp_log (VBI_LOG_ERR,
+		"Invalid VBI scan range %u-%u (%u lines), "
+		"%u-%u (%u lines).",
+		sp->start[0], sp->start[0] + sp->count[0] - 1,
+		sp->count[0],
+		sp->start[1], sp->start[1] + sp->count[1] - 1,
+		sp->count[1]);
 
 	return FALSE;
 }
@@ -659,8 +698,11 @@ _vbi_sampling_par_verify	(const vbi_sampling_par *sp)
 static vbi_bool
 _vbi_sampling_par_check_service	(const vbi_sampling_par *sp,
 				 const _vbi_service_par *par,
-				 int			strict)
+				 int			strict,
+				 vbi_log_fn *		log_fn,
+				 void *			log_user_data)
 {
+	const unsigned int unknown = 0;
 	double signal;
 	unsigned int field;
 	unsigned int samples_per_line;
@@ -681,22 +723,26 @@ _vbi_sampling_par_check_service	(const vbi_sampling_par *sp,
 	}
 
 	if (0 == (par->videostd_set & videostd_set)) {
-		log ("Service 0x%08x (%s) requires "
-		     "videostd_set 0x%08" PRIx64 ", "
-		     "have 0x%08" PRIx64,
-		     par->id, par->label,
-		     par->videostd_set, videostd_set);
+		vbi_log_printf (log_fn, log_user_data,
+				VBI_LOG_NOTICE, __FUNCTION__,
+				"Service 0x%08x (%s) requires "
+				"videostd_set 0x%" PRIx64 ", "
+				"have 0x%" PRIx64,
+				par->id, par->label,
+				par->videostd_set, videostd_set);
 		return FALSE;
 	}
 
-	/* I've seen CC signals on other lines and there's no
-	   way to distinguish from the transmitted data. */
-	if ((VBI_SLICED_CAPTION_525 & par->id)
-	    && (0 == sp->start[0] /* unknown */
-		|| 0 == sp->start[1])) {
-		log ("Service 0x%08x (%s) requires exact line numbers",
-		     par->id, par->label);
-		return FALSE;
+	if (par->flags & _VBI_SP_LINE_NUM) {
+		if ((par->first[0] > 0 && unknown == sp->start[0])
+		    || (par->first[1] > 0 && unknown == sp->start[1])) {
+			vbi_log_printf (log_fn, log_user_data,
+					VBI_LOG_NOTICE, __FUNCTION__,
+					"Service 0x%08x (%s) requires known "
+					"line numbers.",
+					par->id, par->label);
+			return FALSE;
+		}
 	}
 
 	{
@@ -716,10 +762,12 @@ _vbi_sampling_par_check_service	(const vbi_sampling_par *sp,
 		}
 
 		if (rate > (unsigned int) sp->sampling_rate) {
-			log ("Sampling rate %f MHz too low "
-			     "for service 0x%08x (%s)",
-			     sp->sampling_rate / 1e6,
-			     par->id, par->label);
+			vbi_log_printf (log_fn, log_user_data,
+					VBI_LOG_NOTICE, __FUNCTION__,
+					"Sampling rate %f MHz too low "
+					"for service 0x%08x (%s).",
+					sp->sampling_rate / 1e6,
+					par->id, par->label);
 			return FALSE;
 		}
 	}
@@ -741,21 +789,27 @@ _vbi_sampling_par_check_service	(const vbi_sampling_par *sp,
 		end = (sp->offset + samples_per_line) / sampling_rate;
 
 		if (offset > (par->offset / 1e3 - 0.5e-6)) {
-			log ("Sampling offset %f us too late "
-			     "for service 0x%08x (%s) at %f us",
-			     offset * 1e6,
-			     par->id, par->label,
-			     par->offset / 1e3);
+			vbi_log_printf (log_fn, log_user_data,
+					VBI_LOG_NOTICE, __FUNCTION__,
+					"Sampling starts at 0H + %f us, too "
+					"late for service 0x%08x (%s) at "
+					"%f us.",
+					offset * 1e6,
+					par->id, par->label,
+					par->offset / 1e3);
 			return FALSE;
 		}
 
 		if (end < (par->offset / 1e9 + signal + 0.5e-6)) {
-			log ("Sampling ends too early at %f us "
-			     "for service 0x%08x (%s) up to %f us",
-			     end * 1e6,
-			     par->id, par->label,
-			     par->offset / 1e3
-			     + signal * 1e6 + 0.5);
+			vbi_log_printf (log_fn, log_user_data,
+					VBI_LOG_NOTICE, __FUNCTION__,
+					"Sampling ends too early at 0H + "
+					"%f us for service 0x%08x (%s) "
+					"which ends at %f us",
+					end * 1e6,
+					par->id, par->label,
+					par->offset / 1e3
+					+ signal * 1e6 + 0.5);
 			return FALSE;
 		}
 	} else {
@@ -764,19 +818,23 @@ _vbi_sampling_par_check_service	(const vbi_sampling_par *sp,
 		samples = samples_per_line / (double) sp->sampling_rate;
 
 		if (samples < (signal + 1.0e-6)) {
-			log ("Service 0x%08x (%s) signal %f us "
-			     "exceeds %f us sampling period",
-			     par->id, par->label,
-			     signal * 1e6, samples * 1e6);
+			vbi_log_printf (log_fn, log_user_data,
+					VBI_LOG_NOTICE, __FUNCTION__,
+					"Service 0x%08x (%s) signal length "
+					"%f us exceeds %f us sampling length.",
+					par->id, par->label,
+					signal * 1e6, samples * 1e6);
 			return FALSE;
 		}
 	}
 
-	if (!sp->synchronous) {
-		/* Not always, but it's too difficult now to bother. */
-		log ("Service 0x%08x (%s) requires "
-		     "synchronous field order",
-		     par->id, par->label);
+	if ((par->flags & _VBI_SP_FIELD_NUM)
+	    && !sp->synchronous) {
+		vbi_log_printf (log_fn, log_user_data,
+				VBI_LOG_NOTICE, __FUNCTION__,
+				"Service 0x%08x (%s) requires "
+				"synchronous field order.",
+				par->id, par->label);
 		return FALSE;
 	}
 
@@ -794,27 +852,34 @@ _vbi_sampling_par_check_service	(const vbi_sampling_par *sp,
 		}
 
 		if (0 == sp->count[field]) {
-			log ("Service 0x%08x (%s) requires "
-			     "data from field %u",
-			     par->id, par->label, field);
+			vbi_log_printf (log_fn, log_user_data,
+					VBI_LOG_NOTICE, __FUNCTION__,
+					"Service 0x%08x (%s) requires "
+					"data from field %u",
+					par->id, par->label, field + 1);
 			return FALSE;
 		}
 
-		if (sp->start[field] > 0 && strict > 0) {
+		if (strict <= 0 || 0 == sp->start[field])
+			continue;
+
+		if (1 == strict && par->first[field] > par->last[field]) {
 			/* May succeed if not all scanning lines
 			   available for the service are actually used. */
-			if (strict > 1
-			    || (par->first[field] == par->last[field]))
-				if (start > par->first[field]
-				    || end < par->last[field]) {
-					log ("Service 0x%08x (%s) requires "
-					     "lines %u-%u, have %u-%u",
-					     par->id, par->label,
-					     par->first[field],
-					     par->last[field],
-					     start, end);
-					return FALSE;
-				}
+			continue;
+		}
+		
+		if (start > par->first[field]
+		    || end < par->last[field]) {
+			vbi_log_printf (log_fn, log_user_data,
+					VBI_LOG_NOTICE, __FUNCTION__,
+					"Service 0x%08x (%s) requires "
+					"lines %u-%u, have %u-%u.",
+					par->id, par->label,
+					par->first[field],
+					par->last[field],
+					start, end);
+			return FALSE;
 		}
 	}
 
@@ -825,7 +890,9 @@ _vbi_sampling_par_check_service	(const vbi_sampling_par *sp,
 vbi_service_set
 vbi_sampling_par_check_services	(const vbi_sampling_par *sp,
 				 vbi_service_set	services,
-				 int			strict)
+				 int			strict,
+				 vbi_log_fn *		log_fn,
+				 void *			log_user_data)
 {
 	const _vbi_service_par *par;
 	vbi_service_set rservices;
@@ -835,9 +902,12 @@ vbi_sampling_par_check_services	(const vbi_sampling_par *sp,
 	rservices = 0;
 
 	for (par = _vbi_service_table; par->id; ++par) {
-		if (par->id & services)
-			if (_vbi_sampling_par_check_service (sp, par, strict))
-				rservices |= par->id;
+		if (0 == (par->id & services))
+			continue;
+
+		if (_vbi_sampling_par_check_service (sp, par, strict,
+						     log_fn, log_user_data))
+			rservices |= par->id;
 	}
 
 	return rservices;
@@ -847,7 +917,9 @@ vbi_service_set
 vbi_sampling_par_from_services	(vbi_sampling_par *	sp,
 				 unsigned int *		max_rate,
 				 vbi_videostd_set	videostd_set,
-				 vbi_service_set	services)
+				 vbi_service_set	services,
+				 vbi_log_fn *		log_fn,
+				 void *			log_user_data)
 {
 	const _vbi_service_par *par;
 	vbi_service_set rservices;
@@ -859,8 +931,9 @@ vbi_sampling_par_from_services	(vbi_sampling_par *	sp,
 		if (0 == (VBI_VIDEOSTD_SET_ALL & videostd_set)
 		    || ((VBI_VIDEOSTD_SET_525_60 & videostd_set)
 			&& (VBI_VIDEOSTD_SET_625_50 & videostd_set))) {
-			log ("Ambiguous videostd_set 0x%08" PRIx64,
-			     videostd_set);
+			sp_log (VBI_LOG_WARNING,
+				"Ambiguous videostd_set 0x%08" PRIx64,
+				videostd_set);
 			/* CLEAR (*sp); */
 			return 0;
 		}
@@ -906,11 +979,12 @@ vbi_sampling_par_from_services	(vbi_sampling_par *	sp,
 			margin = 2.0e-6;
 
 		if (0 == (par->videostd_set & videostd_set)) {
-			log ("Service 0x%08x (%s) requires "
-			     "videostd_set 0x%08" PRIx64 ", "
-			     "have 0x%08" PRIx64,
-			     par->id, par->label,
-			     par->videostd_set, videostd_set);
+			sp_log (VBI_LOG_NOTICE,
+				"Service 0x%08x (%s) requires "
+				"videostd_set 0x%" PRIx64 ", "
+				"have 0x%" PRIx64,
+				par->id, par->label,
+				par->videostd_set, videostd_set);
 			continue;
 		}
 
@@ -1168,6 +1242,47 @@ add_job_to_pattern		(vbi3_raw_decoder *	rd,
 	return TRUE;
 }
 
+static void
+lines_containing_data		(unsigned int		start[2],
+				 unsigned int		count[2],
+				 const vbi_sampling_par *sp,
+				 const _vbi_service_par *par)
+{
+	unsigned int field;
+
+	start[0] = 0;
+	start[1] = sp->count[0];
+	count[0] = sp->count[0];
+	count[1] = sp->count[1];
+
+	if (!sp->synchronous) {
+		/* XXX Scanning all lines isn't always necessary. */
+		return;
+	}
+
+	for (field = 0; field < 2; ++field) {
+		unsigned int first;
+		unsigned int last;
+
+		if (0 == par->first[field]
+		    || 0 == par->last[field]) {
+			/* No data on this field. */
+			count[field] = 0;
+			continue;
+		}
+
+		first = sp->start[field];
+		last = first + sp->count[field] - 1;
+
+		if (first > 0) {
+			first = MAX (first, (unsigned int) par->first[field]);
+			start[field] += first - sp->start[field];
+			last = MIN (last, (unsigned int) par->last[field]);
+			count[field] = last - first + 1;
+		}
+	}
+}
+
 /**
  * $param rd Pointer to vbi_raw_decoder object allocated with
  *   vbi3_raw_decoder_new().
@@ -1197,16 +1312,19 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 	const _vbi_service_par *par;
 	double min_offset;
 
+	assert (NULL != rd);
+
 	services &= ~(VBI_SLICED_VBI_525 | VBI_SLICED_VBI_625);
 
 	if (rd->services & services) {
-		log ("Already decoding services 0x%08x",
-		     rd->services & services);
+		rd_log (VBI_LOG_INFO,
+			"Already decoding services 0x%08x.",
+			rd->services & services);
 		services &= ~rd->services;
 	}
 
 	if (0 == services) {
-		log ("No services to add");
+		rd_log (VBI_LOG_INFO, "No services to add.");
 		return rd->services;
 	}
 
@@ -1220,28 +1338,28 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 		rd->pattern = (int8_t *)
 			malloc (scan_ways * sizeof (rd->pattern[0]));
 		if (!rd->pattern) {
-			log ("Out of memory");
+			rd_log (VBI_LOG_ERR, "Out of memory.");
 			return rd->services;
 		}
 
 		memset (rd->pattern, 0, scan_ways * sizeof (rd->pattern[0]));
 	}
 
-	if (525 == rd->sampling.scanning)
+	if (525 == rd->sampling.scanning) {
 		min_offset = 7.9e-6;
-	else
+	} else {
 		min_offset = 8.0e-6;
+	}
 
 	for (par = _vbi_service_table; par->id; ++par) {
 		vbi_sampling_par *sp;
 		_vbi3_raw_decoder_job *job;
-		unsigned int j;
-		unsigned int field;
 		unsigned int start[2];
 		unsigned int count[2];
 		unsigned int sample_offset;
 		unsigned int samples_per_line;
 		unsigned int cri_end;
+		unsigned int j;
 
 		if (0 == (par->id & services))
 			continue;
@@ -1265,8 +1383,11 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 		}
 
 		if (j >= _VBI3_RAW_DECODER_MAX_JOBS) {
-			log ("Set 0x%08x exceeds %u service limit",
-			     services, _VBI3_RAW_DECODER_MAX_WAYS);
+			rd_log (VBI_LOG_ERR,
+				"Set 0x%08x exceeds number of "
+				"simultaneously decodable "
+				"services (%u).",
+				services, _VBI3_RAW_DECODER_MAX_WAYS);
 			break;
 		} else if (j >= rd->n_jobs) {
 			job->id = 0;
@@ -1275,7 +1396,8 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 
 		sp = &rd->sampling;
 
-		if (!_vbi_sampling_par_check_service (sp, par, strict))
+		if (!_vbi_sampling_par_check_service
+		    (sp, par, strict, rd->log_fn, rd->log_user_data))
 			continue;
 
 
@@ -1323,40 +1445,13 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 		}
 
 
-		for (field = 0; field < 2; ++field) {
-			unsigned int first;
-			unsigned int last;
-
-			if (0 == par->first[field]
-			    || 0 == par->last[field]) {
-				/* No data on this field. */
-				start[field] = 0;
-				count[field] = 0;
-				continue;
-			}
-
-			first = sp->start[field];
-			last = first + sp->count[field] - 1;
-
-			if (first > 0 && strict > 0) {
-				first = MAX (first,
-					     (unsigned int) par->first[field]);
-				start[field] = first - sp->start[field];
-				last = MIN (last,
-					    (unsigned int) par->last[field]);
-				count[field] = last - first + 1;
-			} else {
-				start[field] = 0;
-				count[field] = sp->count[field];
-			}
-		}
-
-		start[1] += sp->count[0]; /* rel. raw image, not scan lines */
+		lines_containing_data (start, count, sp, par);
 
 		if (!add_job_to_pattern (rd, job - rd->jobs, start, count)) {
-			log ("Out of pattern space "
-			     "for service 0x%08x (%s)",
-			     par->id, par->label);
+			rd_log (VBI_LOG_ERR,
+				"Out of decoder pattern space for "
+				"service 0x%08x (%s).",
+				par->id, par->label);
 			continue;
 		}
 
@@ -1371,6 +1466,14 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 	return rd->services;
 }
 
+vbi_service_set
+vbi3_raw_decoder_services	(vbi3_raw_decoder *	rd)
+{
+	assert (NULL != rd);
+
+	return rd->services;
+}
+
 /**
  * $param rd Pointer to a vbi3_raw_decoder object allocated with
  *   vbi3_raw_decoder_new().
@@ -1403,7 +1506,7 @@ vbi3_raw_decoder_set_sampling_par
 
 	vbi3_raw_decoder_reset (rd);
 
-	if (!_vbi_sampling_par_verify (sp)) {
+	if (!_vbi_sampling_par_valid (sp, rd->log_fn, rd->log_user_data)) {
 		CLEAR (rd->sampling);
 		return 0;
 	}
@@ -1431,6 +1534,22 @@ vbi3_raw_decoder_get_sampling_par
 	*sp = rd->sampling;
 }
 
+void
+vbi3_raw_decoder_set_log_fn	(vbi3_raw_decoder *	rd,
+				 vbi_log_fn *		log_fn,
+				 void *			user_data,
+				 vbi_log_level		max_level)
+{
+	assert (NULL != rd);
+
+	if (NULL == log_fn)
+		max_level = 0;
+
+	rd->log_fn = log_fn;
+	rd->log_user_data = user_data;
+	rd->log_max_level = max_level;
+}
+
 /**
  * $internal
  *
@@ -1458,8 +1577,16 @@ _vbi3_raw_decoder_init		(vbi3_raw_decoder *	rd,
 
 	vbi3_raw_decoder_reset (rd);
 
+	if (RAW_DECODER_LOG) {
+		vbi3_raw_decoder_set_log_fn (rd,
+					     vbi_log_on_stderr,
+					     /* user_data */ NULL,
+					     /* max_level */ VBI_LOG_INFO);
+	}
+
 	if (NULL != sp) {
-		if (!_vbi_sampling_par_verify (sp))
+		if (!_vbi_sampling_par_valid (sp, rd->log_fn,
+					      rd->log_user_data))
 			return FALSE;
 
 		rd->sampling = *sp;
@@ -1505,10 +1632,9 @@ vbi3_raw_decoder_new		(const vbi_sampling_par *sp)
 {
 	vbi3_raw_decoder *rd;
 
-	if (!(rd = malloc (sizeof (*rd)))) {
-		log ("Out of memory");
+	rd = malloc (sizeof (*rd));
+	if (NULL == rd)
 		return NULL;
-	}
 
 	if (!_vbi3_raw_decoder_init (rd, sp)) {
 		free (rd);
