@@ -17,10 +17,10 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: teletext.c,v 1.7 2002/10/22 04:42:40 mschimek Exp $ */
+/* $Id: teletext.c,v 1.21 2006/02/10 06:25:37 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#  include "../config.h"
+#  include "config.h"
 #endif
 
 #include "site_def.h"
@@ -29,6 +29,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <strings.h> /* strncasecmp */
 #include <ctype.h>
 #include <assert.h>
 
@@ -71,11 +72,13 @@ extern const char _zvbi_intl_domainname[];
 #define DEBUG 0
 
 #if DEBUG
-#define printable(c) ((((c) & 0x7F) < 0x20 || ((c) & 0x7F) > 0x7E) ? \
-                      '.' : ((c) & 0x7F))
 #define printv(templ, args...) fprintf(stderr, templ ,##args)
 #else
-#define printv(templ, args...)
+#define printv(templ, args...)						\
+do {									\
+	if (0)								\
+		fprintf(stderr, templ ,##args);				\
+} while (0)
 #endif
 
 #define ROWS			25
@@ -137,7 +140,7 @@ flof_links(vbi_page *pg, vt_page *vtp)
 	for (i = 0; i < COLUMNS + 1; i++) {
 		if (i == COLUMNS || (acp[i].foreground & 7) != col) {
 			for (k = 0; k < 4; k++)
-				if (flof_link_col[k] == col)
+				if ((int) flof_link_col[k] == col)
 					break;
 
 			if (k < 4 && !NO_PAGE(vtp->data.lop.link[k].pgno)) {
@@ -246,11 +249,19 @@ top_label(vbi_decoder *vbi, vbi_page *pg, struct vbi_font_descr *font,
 	return FALSE;
 }
 
+static __inline__ vbi_pgno
+add_modulo			(vbi_pgno		pgno,
+				 int			incr)
+{
+	return ((pgno - 0x100 + incr) & 0x7FF) + 0x100;
+}
+
 static inline void
 top_navigation_bar(vbi_decoder *vbi, vbi_page *pg,
 		   vt_page *vtp)
 {
 	vbi_char ac;
+	vbi_pgno pgno1;
 	int i, got;
 
 	printv("PAGE MIP/BTT: %d\n", vbi->vt.page_info[vtp->pgno - 0x100].code);
@@ -268,14 +279,16 @@ top_navigation_bar(vbi_decoder *vbi, vbi_page *pg,
 	if (pg->page_opacity[1] != VBI_OPAQUE)
 		return;
 
-	for (i = vtp->pgno; i != vtp->pgno + 1; i = (i == 0) ? 0x89a : i - 1)
+	pgno1 = add_modulo (vtp->pgno, 1);
+
+	for (i = vtp->pgno; i != pgno1; i = add_modulo (i, -1))
 		if (vbi->vt.page_info[i - 0x100].code == VBI_TOP_BLOCK ||
 		    vbi->vt.page_info[i - 0x100].code == VBI_TOP_GROUP) {
 			top_label(vbi, pg, pg->font[0], 0, i, 32 + VBI_WHITE, 0);
 			break;
 		}
 
-	for (i = vtp->pgno + 1, got = FALSE; i != vtp->pgno; i = (i == 0x899) ? 0x100 : i + 1)
+	for (i = pgno1, got = FALSE; i != vtp->pgno; i = add_modulo (i, 1))
 		switch (vbi->vt.page_info[i - 0x100].code) {
 		case VBI_TOP_BLOCK:
 			top_label(vbi, pg, pg->font[0], 2, i, 32 + VBI_YELLOW, 2);
@@ -770,25 +783,25 @@ keyword(vbi_link *ld, uint8_t *p, int column,
 		ld->pgno = pgno;
 
 		return i + j;
-	} else if (!strncasecmp(s, "https://", i = 8)) {
+	} else if (!strncasecmp((char *) s, "https://", i = 8)) {
 		ld->type = VBI_LINK_HTTP;
-	} else if (!strncasecmp(s, "http://", i = 7)) {
+	} else if (!strncasecmp((char *) s, "http://", i = 7)) {
 		ld->type = VBI_LINK_HTTP;
-	} else if (!strncasecmp(s, "www.", i = 4)) {
+	} else if (!strncasecmp((char *) s, "www.", i = 4)) {
 		ld->type = VBI_LINK_HTTP;
-		strcpy(ld->url, "http://");
-	} else if (!strncasecmp(s, "ftp://", i = 6)) {
+		strcpy((char *) ld->url, "http://");
+	} else if (!strncasecmp((char *) s, "ftp://", i = 6)) {
 		ld->type = VBI_LINK_FTP;
 	} else if (*s == '@' || *s == 0xA7) {
 		ld->type = VBI_LINK_EMAIL;
-		strcpy(ld->url, "mailto:");
+		strcpy((char *) ld->url, "mailto:");
 		i = 1;
-	} else if (!strncasecmp(s, "(at)", i = 4)) {
+	} else if (!strncasecmp((char *) s, "(at)", i = 4)) {
 		ld->type = VBI_LINK_EMAIL;
-		strcpy(ld->url, "mailto:");
-	} else if (!strncasecmp(s, "(a)", i = 3)) {
+		strcpy((char *) ld->url, "mailto:");
+	} else if (!strncasecmp((char *) s, "(a)", i = 3)) {
 		ld->type = VBI_LINK_EMAIL;
-		strcpy(ld->url, "mailto:");
+		strcpy((char *) ld->url, "mailto:");
 	} else
 		return 1;
 
@@ -826,11 +839,11 @@ keyword(vbi_link *ld, uint8_t *p, int column,
 
 		*back = k;
 
-		strncat(ld->url, s + k, -k);
-		strcat(ld->url, "@");
-		strncat(ld->url, s + i, j);
+		strncat((char *) ld->url, (char *) s + k, -k);
+		strcat((char *) ld->url, "@");
+		strncat((char *) ld->url, (char *) s + i, j);
 	} else
-		strncat(ld->url, s + k, i + j - k);
+		strncat((char *) ld->url, (char *) s + k, i + j - k);
 
 	return i + j;
 }
@@ -925,9 +938,9 @@ vbi_resolve_link(vbi_page *pg, int column, int row, vbi_link *ld)
 
 		if (b <= 0) {
 			if (buffer[j + 1] == ')' && j > 2) {
-				if (!strncasecmp(buffer + j + 1 - 3, "(at", 3))
+				if (!strncasecmp((char *) buffer + j + 1 - 3, "(at", 3))
 					b = j - 3;
-				else if (!strncasecmp(buffer + j + 1 - 2, "(a", 2))
+				else if (!strncasecmp((char *) buffer + j + 1 - 2, "(a", 2))
 					b = j - 2;
 			} else if (buffer[j + 1] == '@' || buffer[j + 1] == 167)
 				b = j;
@@ -1010,6 +1023,8 @@ vbi_page_title(vbi_decoder *vbi, int pgno, int subno, char *buf)
 	ait_entry *ait;
 	int i, j;
 
+	subno = subno;
+
 	if (vbi->vt.top) {
 		for (i = 0; i < 8; i++)
 			if (vbi->vt.btt_link[i].type == 2) {
@@ -1183,7 +1198,8 @@ enhance(vbi_decoder *vbi, vt_magazine *mag, vt_extension *ext,
 	struct pex26 *pt, ptmp;
 	int pdc_hr;
 
-	static void
+	/* XXX nested function not portable, to be removed */
+	void
 	flush(int column)
 	{
 		int row = inv_row + active_row;
@@ -1271,7 +1287,7 @@ enhance(vbi_decoder *vbi, vt_magazine *mag, vt_extension *ext,
 				int raw;
 
 				raw = (row == 0 && i < 9) ?
-					0x20 : vbi_parity(vtp->data.lop.raw[row][i - 1]);
+					0x20 : vbi_unpar8 (vtp->data.lop.raw[row][i - 1]);
 
 				/* set-after spacing attributes cancelling non-spacing */
 
@@ -1289,7 +1305,7 @@ enhance(vbi_decoder *vbi, vt_magazine *mag, vt_extension *ext,
 
 				case 0x0A:		/* end box */
 				case 0x0B:		/* start box */
-					if (i < COLUMNS && vbi_parity(vtp->data.lop.raw[row][i]) == raw) {
+					if (i < COLUMNS && vbi_unpar8 (vtp->data.lop.raw[row][i]) == raw) {
 						printv("... boxed term %d %02x\n", i, raw);
 						mac.opacity = 0;
 					}
@@ -1308,7 +1324,7 @@ enhance(vbi_decoder *vbi, vt_magazine *mag, vt_extension *ext,
 					break;
 
 				raw = (row == 0 && i < 8) ?
-					0x20 : vbi_parity(vtp->data.lop.raw[row][i]);
+					0x20 : vbi_unpar8 (vtp->data.lop.raw[row][i]);
 
 				/* set-at spacing attributes cancelling non-spacing */
 
@@ -1343,7 +1359,8 @@ enhance(vbi_decoder *vbi, vt_magazine *mag, vt_extension *ext,
 		active_column = column;
 	}
 
-	static void
+	/* XXX nested function not portable, to be removed */
+	void
 	flush_row(void)
 	{
 		if (type == OBJ_TYPE_PASSIVE || type == OBJ_TYPE_ADAPTIVE)
@@ -2019,9 +2036,9 @@ enhance(vbi_decoder *vbi, vt_magazine *mag, vt_extension *ext,
 					unicode = vbi_teletext_composed_unicode(
 						p->mode - 0x10, p->data);
 			store:
-					printv("enh row %d col %d print 0x%02x/0x%02x -> 0x%04x %c\n",
+					printv("enh row %d col %d print 0x%02x/0x%02x -> 0x%04x\n",
 						active_row, active_column, p->mode, p->data,
-						gl, unicode & 0xFF);
+						unicode);
 
 					ac.unicode = unicode;
 					mac.unicode = ~0;
@@ -2073,7 +2090,7 @@ post_enhance(vbi_page *pg, int display_rows)
 	for (row = 0; row <= last_row; row++) {
 		for (column = 0; column < COLUMNS; acp++, column++) {
 			if (1)
-				printv("%c", printable(acp->unicode));
+				printv("%c", vbi_printable (acp->unicode));
 			else
 				printv("%04xF%dB%dS%dO%d ", acp->unicode,
 				       acp->foreground, acp->background,
@@ -2299,6 +2316,7 @@ vbi_format_vt_page(vbi_decoder *vbi,
 		struct vbi_font_descr *font;
 		int mosaic_unicodes; /* 0xEE00 separate, 0xEE20 contiguous */
 		int held_mosaic_unicode;
+		int esc;
 		vbi_bool hold, mosaic;
 		vbi_bool double_height, wide_char;
 		vbi_char ac, *acp = &pg->text[row * EXT_COLUMNS];
@@ -2313,6 +2331,7 @@ vbi_format_vt_page(vbi_decoder *vbi,
 		mosaic_unicodes	= 0xEE20; /* contiguous */
 		ac.opacity	= pg->page_opacity[row > 0];
 		font		= pg->font[0];
+		esc		= 0;
 		hold		= FALSE;
 		mosaic		= FALSE;
 
@@ -2327,7 +2346,7 @@ vbi_format_vt_page(vbi_decoder *vbi,
 			if (row == 0 && column < 8) {
 				raw = buf[column];
 				i++;
-			} else if ((raw = vbi_parity(vtp->data.lop.raw[0][i++])) < 0)
+			} else if ((raw = vbi_unpar8 (vtp->data.lop.raw[0][i++])) < 0)
 				raw = ' ';
 
 			/* set-at spacing attributes */
@@ -2403,13 +2422,13 @@ vbi_format_vt_page(vbi_decoder *vbi,
 
 			case 0x0A:		/* end box */
 				if (column < (COLUMNS - 1)
-				    && vbi_parity(vtp->data.lop.raw[0][i]) == 0x0a)
+				    && vbi_unpar8 (vtp->data.lop.raw[0][i]) == 0x0a)
 					ac.opacity = pg->page_opacity[row > 0];
 				break;
 
 			case 0x0B:		/* start box */
 				if (column < (COLUMNS - 1)
-				    && vbi_parity(vtp->data.lop.raw[0][i]) == 0x0b)
+				    && vbi_unpar8 (vtp->data.lop.raw[0][i]) == 0x0b)
 					ac.opacity = pg->boxed_opacity[row > 0];
 				break;
 
@@ -2446,7 +2465,7 @@ vbi_format_vt_page(vbi_decoder *vbi,
 				break;
 
 			case 0x1B:		/* ESC */
-				font = (font == pg->font[0]) ? pg->font[1] : pg->font[0];
+				font = pg->font[esc ^= 1];
 				break;
 			}
 		}
